import { Action } from "../../core/undo/Action";
import { QuestEntityModel } from "../model/QuestEntityModel";
import { entity_data } from "../../core/data_formats/parsing/quest/entities";
import { Euler } from "three";
import { QuestEditorStore } from "../stores/QuestEditorStore";

export class RotateEntityAction implements Action {
    readonly description: string;

    constructor(
        private readonly quest_editor_store: QuestEditorStore,
        private readonly entity: QuestEntityModel,
        private readonly old_rotation: Euler,
        private readonly new_rotation: Euler,
        private readonly world: boolean,
    ) {
        this.description = `Rotate ${entity_data(entity.type).name}`;
    }

    undo(): void {
        this.quest_editor_store.set_selected_entity(this.entity);

        if (this.world) {
            this.entity.set_world_rotation(this.old_rotation);
        } else {
            this.entity.set_rotation(this.old_rotation);
        }
    }

    redo(): void {
        this.quest_editor_store.set_selected_entity(this.entity);

        if (this.world) {
            this.entity.set_world_rotation(this.new_rotation);
        } else {
            this.entity.set_rotation(this.new_rotation);
        }
    }
}
